function     variogram()
% function variogram()
% computes normalized semi-variogram/correlogram model parameter
%% INPUT
%    data.in.tv=f(x,y)		 (x,y) coordinates of transformed variable value
%    para.vario.res   = lag resolution
%    para.vario.range = range limit for semivariogram computation
%    para.vario
%            .angle = orientation angle from horizontal axis in degree
%            .ratio = aspect ratio of the scale in vertical (Y) to that of horizontal (X)
%% OUTPUT
%    data.out.vario.grd =  lag grid
%    data.out.vario.gammah = gamma(h) 	or gamma(grd)   semi-variogram or correlogram
%    data.out.vario.c0  =  variance at each lag
%    data.out.vario.cnt =  no. of pairs at each lag
%

%%  Kriging Software Package  version 2.0,   November 1, 1999
%%  Copyright (c) 1999, property of Dezhang Chu and Woods Hole Oceanographic
%%  Institution.  All Rights Reserved.

global para data
%% data 
if ~isfield(data,'in')
end

x=data.in.x;
y=data.in.y;
var=data.in.tv;

%% parameters
res=para.vario.res;
range=para.vario.max_range;
ang=para.vario.angle;
aspect=para.vario.ratio;

ni=length(var);				% number of data points of observations
indx_nan=find(isnan(var));
var(indx_nan)=[];
x(indx_nan)=[];
y(indx_nan)=[];

nnan=length(indx_nan); 
n=ni-nnan;				% usable number of data points  
if n > 50
  msg1='Computing Semivariogram/Correlogram, please be patient,... ';
  msg2=sprintf('usable data points = %g',n);
  hdl0=message(2,msg1,msg2);
  tic
else
  hdl0=[];
end

if n < 1 
   message(1,'No usable data !!');
   return
end

%% process anisotrophic data
if abs(ang) > eps | abs(1-aspect) > 1e-5
%% observations
    cz=[x(:) y(:)];
    rx=1;
    ry=aspect;
    [cz,rot]=trans2d(cz,ang,rx,ry);
%	 figure;plot(x,y,'dr',cz(:,1),cz(:,2),'og');pause
    x=cz(:,1);y=cz(:,2);
end

m=floor(range/res);
x=reshape(x,n,1);
y=reshape(y,n,1);
var=reshape(var,n,1);

cnt=zeros(m,1);
gamma_sum=zeros(m,1);
var_sum=zeros(m,1);
var2_sum=zeros(m,1);
gammah=zeros(m,1);
c0=zeros(m,1);
if n < 1000
  n_cnt=max(1,round(0.05*n));
else
  n_cnt=max(1,round(0.01*n));
end
head_indx=zeros(n,m);

i0=1;
for i=1:n
	xi=x(i+1:n);yi=y(i+1:n);var_i=var(i+1:n);ni=n-i;
   if rem(i,n_cnt) == 0 & ~isempty(hdl0)
     percent=sprintf('   %g percent done in %g sec.', floor(100*i/n),round(toc));
     message(4,percent,' ',hdl0); 
   end
   d=sqrt( (x(i)-xi).^2+(y(i)-yi).^2);
   [dsort,indx_sort]=sort(d);
   dsort=dsort(i0:ni);								% excluding
   indx_sort=indx_sort(i0:ni);
   var_sort=var_i(indx_sort);       
   dindx=floor(dsort/res)+1;
   k_acc=0;								% accumulated k index
   for k=1:m
     indxk=find(dindx(k_acc+1:ni+1-i0) == k) + k_acc ;
     nk=length(indxk);
     if nk > 0
		% tail portion
        cnt(k)=cnt(k)+nk;
        k_acc=k_acc+nk;
        var_dif= var(i) - var_sort(indxk);	
        var_sum(k)=var_sum(k)+sum(var_sort(indxk));
        var2_sum(k)=var2_sum(k)+var_sort(indxk)'*var_sort(indxk);
        gamma_sum(k)=gamma_sum(k)+ var_dif'*var_dif;
		% head point index
		  head_indx(i,k)=1;
      end
   end
end
sigma0_head=zeros(m,1);
sigma0_tail=zeros(m,1);
for k=1:m
 	indx1=find(head_indx(:,k) == 1);
	if ~isempty(indx1)
		sigma0_head(k)=std_nan(var(indx1));
	end
end

indx=find( cnt >= 2);
if length(indx) > 0
   cnt_nz=cnt(indx);
   var_mean=var_sum(indx)./cnt_nz;
   sigma0_tail(indx)=sqrt(abs(var2_sum(indx)./cnt_nz-var_mean.^2));
   c0(indx)=sigma0_tail(indx).*sigma0_head(indx);
   gammah(indx)=0.5*gamma_sum(indx)./(cnt_nz.*c0(indx)+eps);
else
   gammah=nan.*ones(m,1);
end
indx=find(gammah > para.vario.max_value);
gammah(indx)=nan*ones(size(indx));

data.out.vario.lag=[0:res:res*m]';	% grid for plotting
data.out.vario.gammah=[0;gammah];
data.out.vario.cnt=[0;cnt(:)];
indx1=find(c0 > 0);
data.out.vario.c0_mean=median(c0(indx1));
data.out.vario.c0=max(c0(indx1));
close(hdl0);