function     variogram2d()
% function variogram2d()
% computes normalized 2d semi-variogram/correlogram model parameter
%% INPUT
%    data.in.tv=f(x,y)		 (x,y) coordinates of transformed variable value
%    para.vario.res   = lag resolution
%    para.vario.range = range limit for semivariogram computation
%    para.vario
%            .angle = orientation angle from horizontal axis in degree
%            .ratio = aspect ratio of the scale in vertical (Y) to that of horizontal (X)
%% OUTPUT
%    data.out.vario.grd2d =  angle-lag grid
%    data.out.vario.gammah2d = gamma(h) 	or gamma(grd)   semi-variogram or correlogram
%

%%  Kriging Software Package  version 2.0,   November 1, 1999
%%  Copyright (c) 1999, property of Dezhang Chu and Woods Hole Oceanographic
%%  Institution.  All Rights Reserved.

global para data


%% data 
x0=data.in.x;
y0=data.in.y;
var0=data.in.tv;
ni=length(var0);				% number of data points of observations
indx_nan=find(isnan(var0));
var0(indx_nan)=[];
x0(indx_nan)=[];
y0(indx_nan)=[];
nnan=length(indx_nan); 
n=ni-nnan;				% usable number of data points  

if n < 1 
   message(1,'No usable data !!');
   return
end

% initialization
data.out.vario.gammah2d=[];
data.out.vario.x=[];
data.out.vario.y=[];


%% parameters
res=para.vario.res;
range=para.vario.disp_range;
m=floor(range/res);

ang_res=para.vario.ang_res;
angle=[0:ang_res:180-ang_res];
lag=0:res:res*m;
l=length(angle);

msg1='Computing 2-D Semi-variogram/Correlogram, please be patient,... ';
msg2=sprintf('usable data points for each angle = %g',n);
hdl0=message(2,msg1,msg2);
tic

aspect=para.vario.ratio;
rx=1;
i0=1;
nl_cnt=max(1,round(0.01*n*l));

%% update data based on sepecified anisotropy parameters
ang=para.vario.angle;
cz=[x0(:) y0(:)];
ry=aspect;							% aspect = ry/rx = ry since rx is set to 1
[cz,rot]=trans2d(cz,ang,rx,ry);
x0=cz(:,1);y0=cz(:,2);
ii=sqrt(-1);

for j=1:l						% orientation loop
 ang=angle(j);
%% 2d-coordinates transformation
% initialization
 cnt=zeros(m,1);
 gamma_sum=zeros(m,1);
 var_sum=zeros(m,1);
 var2_sum=zeros(m,1);
 gammah=nan*zeros(m,1);
 c0=zeros(m,1);
 head_indx=zeros(n,m);
 ss=[];
 for i=1:n					% sample loop
	ij=(j-1)*n+i;
   if rem(ij,nl_cnt) == 0 & ~isempty(hdl0)
     percent=sprintf('%g percent done in %g sec.', floor(100*ij/(l*n)),round(toc));
     message(4,percent,' ',hdl0); 
   end
   phs=atan2(y0(i+1:n)-y0(i),x0(i+1:n)-x0(i))*180/pi;
	neg_indx=find(phs < ang - 0.5*ang_res);
   if ~isempty(neg_indx) phs(neg_indx)=180+phs(neg_indx);end				
   phs_indx=find(phs >= ang - 0.5*ang_res & phs < ang+0.5*ang_res)+i;
   if ~isempty(phs_indx)
	xi=x0(phs_indx);yi=y0(phs_indx);var_i=var0(phs_indx);
%plot(x0(i+1:n),y0(i+1:n),'.k',x0(phs_indx),y0(phs_indx),'or');pause
   ni=length(xi);
   d=sqrt( (x0(i)-xi).^2+(y0(i)-yi).^2);
   [dsort,indx_sort]=sort(d);
   dsort=dsort(i0:ni);								% excluding
   indx_sort=indx_sort(i0:ni);
   var_sort=var_i(indx_sort);       
   dindx=floor(dsort/res)+1;
   k_acc=0;								% accumulated k index
   for k=1:m
     indxk=find(dindx(k_acc+1:ni+1-i0) == k) + k_acc ;
     nk=length(indxk);
     if nk > 0
		% tail portion
        cnt(k)=cnt(k)+nk;
        k_acc=k_acc+nk;
        var_dif= var0(i) - var_sort(indxk);	
        var_sum(k)=var_sum(k)+sum(var_sort(indxk));
        var2_sum(k)=var2_sum(k)+var_sort(indxk)'*var_sort(indxk);
        gamma_sum(k)=gamma_sum(k)+ var_dif'*var_dif;
		% head point index
		  head_indx(i,k)=1;
			if j==-11 & k == 35
				disp([j i k])
				disp([ var_sum(k) var2_sum(k) cnt(k) (var_sum(k)/cnt(k))^2 var2_sum(k)/cnt(k) ])
				ss=[ss; var_sort(indxk)];
				disp([indxk(:) var_sort(indxk)'])
         end
      end
   end					% end of lag loop
	end					% not empty 
 end					% end of selected observation data loop		
 sigma0_head=zeros(m,1);
 sigma0_tail=zeros(m,1);
 for k=1:m
 	indx1=find(head_indx(:,k) == 1);
	if ~isempty(indx1)
		sigma0_head(k)=std_nan(var0(indx1));
	end
 end

 indx=find( cnt >= 2);
 if length(indx) > 0
   cnt_nz=cnt(indx);
   var_mean=var_sum(indx)./cnt_nz;
   var_std2=var2_sum(indx)./cnt_nz-var_mean.^2;
   sigma0_tail(indx)=sqrt(abs(var_std2));
   c0(indx)=sigma0_tail(indx).*sigma0_head(indx);
   gammah(indx)=0.5*gamma_sum(indx)./(cnt_nz.*c0(indx)+eps);
 else
   gammah=nan.*ones(m,1);
 end
 indx=find(gammah > para.vario.max_value);
 gammah(indx)=nan*ones(size(indx));
 data.out.vario.gammah2d(j,1:m+1)=[0;gammah]';
 data.out.vario.x(j,:)=cos((ang-0.5*ang_res)*pi/180)*lag;
 data.out.vario.y(j,:)=sin((ang-0.5*ang_res)*pi/180)*lag;
 indxw=find(data.out.vario.gammah2d > 0);
% disp(sprintf('ang = %g, non-zero elements = %g',ang,length(indxw)))
end									% angle of orientation loop
data.out.vario.lag=lag(:);	% grid for plotting
data.out.vario.ang=angle(:);	

close(hdl0);